//*****************************************************************************
// Filename:
//    wc_globals.cpp
// 
// Description:
//    This file contains function global to WebCatalog.
// 
//*****************************************************************************

#include <stdlib.h>
#include <errno.h>

#ifndef _WINNTCGI
#include <syslog.h>
#include <strings.h>
#include <unistd.h>
#endif

#include <string.h>

#include "wc_globals.h"

//*****************************************************************************
// Function:
//    safe_strcpy
//
// Description:
//    This function copies a string truncating it if it exceeds a maximum
//    length.
//
// Arguments:
//    dest (in/out) - The copy of src.
//    src      (in) - The string to copy.
//    max_len  (in) - The maximum length to copy to dest.
//
// Returns: 
//    0 - If src was truncated.
//    1 - If src was not truncated.
//   -1 - If either dest or src is NULL.
//
//*****************************************************************************
int safe_strcpy(
   char *dest,
   char *src,
   size_t max_len
)
{
   int ret_value = -1;

   ENTER_TRACE("safe_strcpy");
   
   if (!dest || !src)
   {
      return ret_value;
   }
   
   if (strlen(src) > max_len)
   {
      ret_value = 0;
   }
   else
   {
      ret_value = 1;
   }
   
   strncpy(dest, src, max_len);
   dest[max_len] = '\0';
   
   EXIT_TRACE("safe_strcpy");
   
   return ret_value;

} // end safe_strcpy

//*****************************************************************************
// Function:
//    write_string
//
// Description:
//    This function appends a string to the end of a buffer. It allocates
//    more memory if the buffer is not big enough for the string.
//
// Arguments:
//    buffer   (in/out) - The pointer to the buffer we are appending 
//                        the string to. This value could change if more
//                        memory is allocated.
//    curr_ptr (in/out) - Pointer to the beginning of the next available byte 
//                        in the buffer. This value changes every time.
//    string   (in)     - String to be added to the buffer.
//    buf_size (in/out) - The current size of the buffer. This value changes
//                        if more memory is allocated.
//
// Returns: 
//    1 - Success
//   -1 - Failure
//
//*****************************************************************************
int write_string (
   char **buffer,
   char **cur_ptr,
   char *string,
   size_t &buf_size
)
{
   size_t space_used = 0;

   //
   // Make sure non of the pointer 
   // passed in is NULL.
   //
   if (!buffer || !cur_ptr)
   {
      return -1;
   }
   
   //
   // If the string is a NULL pointer, 
   // make it point to an empty string.
   //
   if (!string)
   {
      string = (char*)"";
   }

   //
   // If the size of the buffer is zero
   // create the initial buffer and set 
   // the size accordingly.
   //
   if (!buf_size)
   {
      *buffer = (char *)malloc(INIT_SIZE);
      *cur_ptr = *buffer;
      buf_size = INIT_SIZE;
   }

   //
   // If the remaining space left on the buffer
   // is not big enough to fit our string, increment
   // the buffer size by INC_SIZE until it is.
   //
   space_used = *cur_ptr - *buffer;
   while ((strlen(string) + 1) > (buf_size - space_used))
   {
      buf_size += INC_SIZE;
      *buffer = (char *)realloc(*buffer, buf_size);
      *cur_ptr = *buffer + space_used;
   }
   
   //
   // Copy the string into the buffer
   // moving the current pointer forward.
   //
   strcpy(*cur_ptr, string);
   *cur_ptr += strlen(string) + 1;
   
   return 1;

} // end write_string

//*****************************************************************************
// Function:
//    append_to_last_string
//
// Description:
//    This function appends a string to the last string in the buffer, 
//    allocating more memory if the buffer is not big enough for the string.
//
// Arguments:
//    buffer   (in/out) - The pointer to the buffer we are appending 
//                        the string to. This value could change if more
//                        memory is allocated.
//    curr_ptr (in/out) - Pointer to the beginning of the next available byte 
//                        in the buffer. This value changes every time.
//    string   (in)     - String to be appended to the last string.
//    buf_size (in/out) - The current size of the buffer. This value changes
//                        if more memory is allocated.
//
// Returns:
//    1 - Success
//   -1 - Failure
//
//*****************************************************************************
int append_to_last_string (
   char **buffer,
   char **cur_ptr,
   char *string,
   size_t &buf_size
)
{
   size_t space_used = 0;

   //
   // Make sure non of the pointer 
   // passed in is NULL.
   //
   if (!buffer || !cur_ptr)
   {
      return -1;
   }
   
   //
   // Set string to point to an
   // empty string, if it is NULL.
   //
   if (!string)
   {
      string = (char*)"";
   }

   //
   // If the size of the buffer is zero
   // create the initial buffer and set 
   // the size accordingly.
   //
   if (!buf_size)
   {
      *buffer = (char *)malloc(INIT_SIZE);
      *cur_ptr = *buffer;
      buf_size = INIT_SIZE;
   }

   //
   // If the remaining space left on the buffer
   // is not big enough to fit our string, increment
   // the buffer size by INC_SIZE until it is.
   //
   space_used = *cur_ptr - *buffer;
   while (strlen(string) > (buf_size - space_used))
   {
      buf_size += INC_SIZE;
      *buffer = (char *)realloc(*buffer, buf_size);
      *cur_ptr = *buffer + space_used;
   }
   
   //
   // Copy the string into the buffer
   // moving the current pointer forward.
   //
   if (*cur_ptr != *buffer)
   {
      (*cur_ptr)--;
   }
   strcpy(*cur_ptr, string);
   *cur_ptr += strlen(string) + 1;
   return 1;
}

//*****************************************************************************
// Function:
//    write_mem
//
// Description:
//    This function appends a block of memory to the end of a buffer. It 
//    allocates more memory if the buffer is not big enough for the block.
//
// Arguments:
//    buffer   (in/out) - The pointer to the buffer we are appending 
//                        the string to. This value could change if more
//                        memory is allocated.
//    curr_ptr (in/out) - Pointer to the beginning of the next available byte.
//                        This value changes every time.
//    string       (in) - pointer to the memory to be added to the buffer.
//    block_size   (in) - The size of the memory block to be added.
//    buf_size (in/out) - The current size of the buffer. This value changes
//                        if more memory is allocated.
//
// Returns:
//    1 - Success
//   -1 - Failure
//
//*****************************************************************************
int write_mem (
   char **buffer,
   char **cur_ptr,
   char *string,
   size_t block_size,
   size_t &buf_size
)
{
   size_t space_used = 0;

   //
   // Make sure non of the pointer 
   // passed in is NULL.
   //
   if (!buffer || !cur_ptr || !string)
   {
      return -1;
   }

   //
   // If the size of the buffer is zero
   // create the initial buffer and set 
   // the size accordingly.
   //
   if (!buf_size)
   {
      *buffer = (char *)malloc(INIT_SIZE);
      *cur_ptr = *buffer;
      buf_size = INIT_SIZE;
   }

   //
   // If the remaining space left on the buffer
   // is not big enough to fit our string, increment
   // the buffer size by INC_SIZE until it is.
   //
   space_used = *cur_ptr - *buffer;
   while (block_size > (buf_size - space_used))
   {
      buf_size += INC_SIZE;
      *buffer = (char *)realloc(*buffer, buf_size);
      *cur_ptr = *buffer + space_used;
   }
   
   //
   // Copy the memory block into the buffer
   // moving the current pointer forward.
   //
   memcpy(*cur_ptr, string, block_size);
   *cur_ptr += block_size;
   return 1;
}



#ifndef _WINNTCGI
//*****************************************************************************
// Function:
//    socket_read
//
// Description:
//    This function tries to read as many bytes requested from a socket
//    file descriptor handling a signal interrupt.
//
// Arguments:
//    fd   (in) - Socket file descriptor to read from.
//    vptr (in) - Buffer where data read is stored.
//    n    (in) - Number of bytes to try to read.
//
// Returns:
//    0 - Socket connection was closed by peer.
//   >0 - Number of bytes read.
//   <0 - Failure
//
//*****************************************************************************
ssize_t socket_read(
   int fd, 
   void *vptr,
   size_t n
)
{
   size_t nleft;
   ssize_t nread;
   char *ptr;

   ptr = (char*)vptr;
   nleft = n;
   while (nleft > 0)
   {
      if ((nread = read(fd, ptr, nleft)) < 0)
      {
         if (errno == EINTR)
         {
            nread = 0;
         }
         else
         {
            return (-1);
         }
      }
      else if (nread == 0)
      {
         break;
      }
      nleft -= nread;
      ptr += nread;
   } // end while

   //
   // Return the number of bytes read.
   //
   return (n - nleft);
} // end socket_read

//*****************************************************************************
// Function:
//    socket_write
//
// Description:
//    This function tries to write as many bytes requested to a socket
//    file descriptor handling a signal interrupt.
//
// Arguments:
//    fd   (in) - Socket file descriptor to write to.
//    vptr (in) - Data to write.
//    n    (in) - Number of bytes to read.
//
// Returns:
//    0 - Socket connection was closed by peer.
//   >0 - Number of bytes written.
//   <0 - Failure
//
//*****************************************************************************
ssize_t socket_write(
   int fd, 
   const void *vptr,
   size_t n
)
{
   size_t nleft;     // number of bytes left to write
   ssize_t nwritten; // bytes written
   const char *ptr;  // pointer to the data to write
   
   ptr = (char*)vptr; 
   nleft = n;
   
   //
   // While there is still data to write.
   //
   while(nleft > 0)
   {
      //
      // Try to write however many 
      // bytes are left to write.
      //
      if ((nwritten = write(fd, ptr, nleft)) <= 0)
      {
         // 
         // If the call to write returned an error
         // check to see if it was an interrupt by
         // a signal, if it is continue writing,
         // else return an error. 
         //
         if (errno == EINTR)
         {
            nwritten = 0;
         }
         else
         {
            return (-1);
         }
      }
      //
      // Decrement nleft by however many
      // bytes were written and advance ptr.
      //
      nleft -= nwritten;
      ptr += nwritten;
   } // end while
   
   //
   // Return the bytes written.
   //
   return n;
} // end socket_write


//*****************************************************************************
// Function:
//    LogError
//
// Description:
//    This function logs an error message to the system error log.
//
// Arguments:
//		error_str (in) - String to be sent the the error log.
//
// Returns: 
//    None.
//
//*****************************************************************************
void LogError(
	char *error_str
)
{
	ENTER_TRACE("LogError");

	openlog("WebDNA", LOG_PID, LOG_DAEMON);
	//syslog(LOG_ERR, "%s - %m", error_str);
	syslog(LOG_ERR, "%s ", error_str);
	closelog();
	EXIT_TRACE("LogError");
} // end LogError



#endif // ifndef _WINNTCGI




//*****************************************************************************
// Function:
//    GetNextString
//
// Description:
//    This function returns a pointer to the next string in the list.
//
// Arguments:
//    buff_ptr (in) - Pointer the the buffer that contains the list of strings.
//
// Returns: 
//    The pointer to the next string or NULL if buff_ptr is NULL.
//
//*****************************************************************************
char * GetNextString(
   char *buff_ptr
)
{
   char *temp_ptr = NULL;  // pointer to the next string on the list
   
   ENTER_TRACE("GetNextString");
   
   if (!buff_ptr)
   {
      return NULL;
   }

   //
   // Get the pointer to the next string. 
   //
   temp_ptr = buff_ptr + strlen(buff_ptr) + 1;

   EXIT_TRACE("GetNextString");

   return temp_ptr; 

} // end GetNextString


